body {
    font-size: 12px;
    font-family: sans-serif;
}

a {
    color: inherit;
}

p {
    font-size: 12px;
    font-family: sans-serif;
    color: #000000;
    margin-bottom: 1em;
}

img {
    border: none;
}

code {
    white-space: pre;
    display: block;
}

div.footer {
    margin-top: 10px;
    font-size: 10px;
}

/* Hide unimportant stuff */
fieldset,
div.changelog,
ul.navigation {
    display: none;
}
