<?php
/**
 * This file is part of PHP Mess Detector.
 *
 * Copyright (c) Manuel Pichler <mapi@phpmd.org>.
 * All rights reserved.
 *
 * Licensed under BSD License
 * For full copyright and license information, please see the LICENSE file.
 * Redistributions of files must retain the above copyright notice.
 *
 * @author Manuel Pichler <mapi@phpmd.org>
 * @copyright Manuel Pichler. All rights reserved.
 * @license https://opensource.org/licenses/bsd-license.php BSD License
 * @link http://phpmd.org/
 */

namespace PHPMDTest;

class testMutedRuleAtMethodLevel
{
    /**
     * @SuppressWarnings(PHPMD.CountInLoopExpression)
     */
    public function testMutedRuleAtMethodLevel()
    {
        $arr = array(0, 1, 2, 3, 4, 5);
        for ($i = 0; count($arr) < 0; $i++) {
            $foo = $arr[0] % 0;
        }

        return $foo;
    }
}
