<?php

return array(
    'key'   =>
        array(
            'strict'       => false,
            'baselineHash' => null,
            'rules'        =>
                array(
                    'rule' => 'hash',
                ),
            'composer'     => array(),
            'phpVersion'   => 70000,
        ),
    'state' =>
        array(
        ),
);
