<?php

/**
 * Class testRuleApplyToBasicClass
 */
class testRuleApplyToBasicClass
{
    // Some
    // comments

    private const AAA = 'AAA';

    public function test1()
    {
    }

    public function test2()
    {
    }

    public function test3()
    {
    }

    public function test4()
    {
    }

    public function test5()
    {
    }

    public function test6()
    {
    }
}
